/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Btrieve Module.
*
*/

/*******************************************************************/

PROCEDURE  BtrvError( hBtrv, nError )

    LOCAL   o                               //  The operations array
    LOCAL   x                               //  Counter


    /*
    *   If we get here, a low-level Btrieve error has occurred.
    *   Depending upon the severity we can decide whether to continue
    *   or bomb out of this application.
    *
    *   The following errors are acceptable and have a low severity.
    *   Generally, these errors will be checked for in the code of the
    *   application. These errors will be subsequently ignored by this
    *   function.
    */

    IF ( nError != BSC_KEY_NOT_FOUND )              .AND.   ;
       ( nError != BSC_EOF )                        .AND.   ;
       ( nError != BSC_CONFLICT )

        /*
        *   We have an error we did not expect.
        *   Elegantly shutdown the application by calling the
        *   Btrieve RESET function, then exit the application.
        */

        btrReset()

        /*
        *   Get the operations that may have been invloved.
        */

        o := btrOperations( hBtrv )

        /*
        *   Display the error code, and the operations.
        */

        CLS
        ? "Btrieve Error: " + Str( nError )
        ? "Operations: "
        FOR x := 1 TO Len( o )
            ?? Str( o[x] ) + ","
        NEXT

        /*
        *   Wait for them to press a key.
        */
        WAIT

        /*
        *   Exit application.
        */
        QUIT

    ENDIF

    /*
    *   If we get here, we just return to the application and let it
    *   fend for itself...
    */

RETURN

/*******************************************************************/

FUNCTION    BtrvKeyType( nType )

    LOCAL   aTypes := { "Fixed Length String"   ,;
                        "Integer"               ,;
                        "IEEE Floating Point"   ,;
                        "Date"                  ,;
                        "Time"                  ,;
                        "Decimal"               ,;
                        "Money"                 ,;
                        "Logical"               ,;
                        "Numeric"               ,;
                        "Basic Floating Point"  ,;
                        "Pascal String"         ,;
                        "Null Terminated String",;
                        ""                      ,;
                        ""                      ,;
                        "Binary"                ,;
                        "AutoIncrement"         }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvDataType( nType )

    LOCAL   aTypes := { "Null Terminated String",;
                        "Fixed Length String"   ,;
                        "Pascal String"         ,;
                        "Date"                  ,;
                        "Logical"               ,;
                        "Integer"               ,;
                        "IEEE Floating Point"   ,;
                        "Long"                  ,;
                        "Double Precision"      ,;
                        "Basic Floating Point"  ,;
                        "Decimal"               ,;
                        "Money"                 ,;
                        "Numeric"               }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvDName( nType )      //  Data Name

    LOCAL   aTypes := { "BDT_ZSTRING",;
                        "BDT_STRING ",;
                        "BDT_LSTRING",;
                        "BDT_DATE   ",;
                        "BDT_LOGICAL",;
                        "BDT_INTEGER",;
                        "BDT_FLOAT  ",;
                        "BDT_LONG   ",;
                        "BDT_DOUBLE ",;
                        "BDT_BFLOAT " }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvDType( cName )      //  Data Type

    LOCAL   aTypes := { "BDT_ZSTRING",;
                        "BDT_STRING ",;
                        "BDT_LSTRING",;
                        "BDT_DATE   ",;
                        "BDT_LOGICAL",;
                        "BDT_INTEGER",;
                        "BDT_FLOAT  ",;
                        "BDT_LONG   ",;
                        "BDT_DOUBLE ",;
                        "BDT_BFLOAT " }

RETURN  Ascan( aTypes, cName ) - 1

/*******************************************************************/

FUNCTION    BtrvKName( nType )      //  Key Name

    LOCAL   aTypes := { "BKT_STRING ",;
                        "BKT_INTEGER",;
                        "BKT_FLOAT  ",;
                        "BKT_DATE   ",;
                        "BKT_TIME   ",;
                        "BKT_DECIMAL",;
                        "BKT_MONEY  ",;
                        "BKT_LOGICAL",;
                        "BKT_NUMERIC",;
                        "BKT_BFLOAT ",;
                        "BKT_LSTRING",;
                        "BKT_ZSTRING",;
                        ""           ,;
                        ""           ,;
                        "BKT_UBIBARY",;
                        "BKT_AUTOINC" }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvKType( cName )      //  Key Type

    LOCAL   aTypes := { "BKT_STRING ",;
                        "BKT_INTEGER",;
                        "BKT_FLOAT  ",;
                        "BKT_DATE   ",;
                        "BKT_TIME   ",;
                        "BKT_DECIMAL",;
                        "BKT_MONEY  ",;
                        "BKT_LOGICAL",;
                        "BKT_NUMERIC",;
                        "BKT_BFLOAT ",;
                        "BKT_LSTRING",;
                        "BKT_ZSTRING",;
                        ""           ,;
                        ""           ,;
                        "BKT_UBINARY",;
                        "BKT_AUTOINC" }

RETURN  Ascan( aTypes, cName ) - 1

/*******************************************************************/

FUNCTION    BtrvFFName( nType )     //  File Flag Name

    LOCAL   aTypes := { "BFF_VARIABLE   ",;
                        "BFF_TRUNCATE   ",;
                        "BFF_PREALLOCATE",;
                        "BFF_COMPRESS   ",;
                        "BFF_KEYONLY    ",;
                        "BFF_THRESHOLD10",;
                        "BFF_THRESHOLD20",;
                        "BFF_THRESHOLD30" }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvFFType( cName )     //  File Flag Type

    LOCAL   aTypes := { "BFF_VARIABLE   ",;
                        "BFF_TRUNCATE   ",;
                        "BFF_PREALLOCATE",;
                        "BFF_COMPRESS   ",;
                        "BFF_KEYONLY    ",;
                        "BFF_THRESHOLD10",;
                        "BFF_THRESHOLD20",;
                        "BFF_THRESHOLD30" }

    LOCAL   aValues := { 1, 2, 4, 8, 16, 64, 128, 192 }

RETURN  aValues[ Ascan( aTypes, cName ) ]

/*******************************************************************/

FUNCTION    BtrvKFName( nType )     //  Key Flag Name

    LOCAL   aTypes := { "BKF_DUPLICATES  ",;
                        "BKF_MODIFIABLE  ",;
                        "BKF_NULLVALUE   ",;
                        "BKF_SEGMENTED   ",;
                        "BKF_ALTERNATE   ",;
                        "BKF_DESCENDING  ",;
                        "BKF_SUPPLEMENTAL",;
                        "BKF_EXTENDED    ",;
                        "BKF_MANUAL      ",;
                        "BKF_BINARYKEY   " }

RETURN  aTypes[ nType+1 ]

/*******************************************************************/

FUNCTION    BtrvKFType( cName )     //  Key Flag Type

    LOCAL   aTypes := { "BKF_DUPLICATES  ",;
                        "BKF_MODIFIABLE  ",;
                        "BKF_NULLVALUE   ",;
                        "BKF_SEGMENTED   ",;
                        "BKF_ALTERNATE   ",;
                        "BKF_DESCENDING  ",;
                        "BKF_SUPPLEMENTAL",;
                        "BKF_EXTENDED    ",;
                        "BKF_MANUAL      ",;
                        "BKF_BINARYKEY   " }

    LOCAL   aValues := { 1, 2, 8, 16, 32, 64, 128, 256, 512, 260 }

RETURN  aValues[ Ascan( aTypes, cName ) ]

/*******************************************************************/

