/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Function Module.
*
*/

/*******************************************************************/

PROCEDURE   HelpLine( cHelp )

    STATIC  iaHelp := {}
    LOCAL   nRow := Row()
    LOCAL   nCol := Col()


    IF ( cHelp == NIL )

        Asize( iaHelp, Len( iaHelp ) - 1 )

        IF ( ! Empty( iaHelp ) )

            cHelp := Atail( iaHelp )

        ENDIF

    ELSE

        Aadd( iaHelp, cHelp )

    ENDIF

    IF ( cHelp != NIL )

        SetPos( Pub( PUB_MAXROW ), 0 )
        DispOut( Replicate( " ", Pub( PUB_MAXCOL )+1 ), Pub( PUB_CLR_HELPLINE ) )

        SetPos( Pub( PUB_MAXROW ), (Pub( PUB_MAXCOL ) - Len( cHelp )) / 2 )
        DispOut( cHelp, Pub( PUB_CLR_HELPLINE ) )

        SetPos( nRow, nCol )

    ENDIF

RETURN

/*******************************************************************/

PROCEDURE   Msg( nRow, nCol, cTitle, cMsg )

    LOCAL   aMsg := Array(0)
    LOCAL   nLen
    LOCAL   nPos
    LOCAL   aScreen
    LOCAL   x
    LOCAL   nCursor
    LOCAL   nWidth := 0


    /*
    *   Turn the cursor off.
    */
    nCursor := SetCursor( SC_NONE )

    /*
    *   Extract all lines, delimited by semicolon.
    */

    WHILE ( ( nPos := At( ";", cMsg ) ) > 0 )

        Aadd( aMsg, Left( cMsg, nPos-1 ) )
        cMsg := Right( cMsg, Len( cMsg ) - nPos )

        IF ( nWidth < nPos-1 )
            nWidth := nPos-1
        ENDIF

    ENDDO

    /*
    *   Add the final line.
    *   If no delimiters, this is the only line.
    */

    Aadd( aMsg, cMsg )

    IF ( nWidth < Len( cMsg ) )
        nWidth := Len( cMsg )
    ENDIF

    /*
    *   Revise the new length.
    */

    nLen := Len( aMsg )

    /*
    *   Check width against title.
    */

    IF ( nWidth < Len( cTitle ) )
        nWidth := Len( cTitle )
    ENDIF

    /*
    *   Display a Standard Menu Box.
    */

    aScreen := MenuBox( cTitle, nRow, nCol, nLen , nWidth )

    /*
    *   Display the message.
    */

    FOR x := 1 TO nLen

        SetPos( nRow+2+x, nCol+3 )
        DispOut( Replicate( " ", nWidth ), Pub( PUB_CLR_BOX_INNER ) )

        SetPos( nRow+2+x, nCol+3 )
        DispOut( aMsg[x], Pub( PUB_CLR_BOX_INNER ) )

    NEXT

    /*
    *   Display the help.
    */

    HelpLine( "Press Any Key To Continue" )

    /*
    *   Wait till we get a keypress.
    */

    Inkey(0)

    /*
    *   Restore cursor.
    */

    SetCursor( nCursor )

    /*
    *   Release the screen.
    */

    RestScreen( aScreen[1], aScreen[2], aScreen[3], aScreen[4], aScreen[5] )

    /*
    *   Restore the old help.
    */

    HelpLine()

RETURN

/*******************************************************************/

FUNCTION    SelectBfm( nRow, nCol, cDir )

    LOCAL   aDir
    LOCAL   x
    LOCAL   aMenu
    LOCAL   cFile := NIL


    /*
    *   Check for a backslash.
    */

    IF( ! Empty(cDir) )
        cDir += "\"
    ENDIF

    /*
    *   Get a list of BFM files.
    */

    aDir := Directory( cDir + "*.BFM" )
    aMenu := Array(0)

    /*
    *   Sort by filename.
    */

    Asort( aDir,,, {|a,b| a[ F_NAME ] < b[ F_NAME ]} )

    /*
    *   Fix up for menu.
    */

    FOR x := 1 TO Len( aDir )

        Aadd( aMenu, { aDir[ x,F_NAME ],;
                     {|| TRUE },;
                     {|n| cFile := aDir[n,1], SimKey(K_ESC)} } )

    NEXT

    /*
    *   Display the files for selection.
    */

    IF ( ! Empty( aDir ) )

        Menu( nRow, nCol, "BFM Files", "ENTER To Select, ESC To Exit", 1, 7 , aMenu )

    ELSE

        Msg( nRow, nCol, "File Not Found", "No BFM files were located in;" + cDir )

    ENDIF

RETURN  IIF(cFile==NIL,NIL,cDir + cFile)

/*******************************************************************/

PROCEDURE   SimKey( nKey )

    KEYBOARD Chr(nKey)

RETURN

/*******************************************************************/

FUNCTION    FileLine( h, cSearch, lGoTop )

    LOCAL   cLine := NIL
    LOCAL   cBuf
    LOCAL   nLen := Len( cSearch )


    /*
    *   Rewind.
    */

    IF ( lGoTop )

        Fseek( h, 0, FS_SET )

    ENDIF

    /*
    *   Search the file.
    */

    WHILE ( ! Feof( h ) )

        cBuf := FreadLine( h )

        IF ( Lower(Left( cBuf, nLen )) == Lower(cSearch) )

            cLine := cBuf
            EXIT

        ENDIF

    ENDDO

RETURN  cLine

/*******************************************************************/

