/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Menu Module.
*
*/
/*******************************************************************/

PROCEDURE   Menu( nRow, nCol, cTitle, cHelp, nSelect, nMaxOptions, aOptions )

    LOCAL   nLen
    LOCAL   aScreen
    LOCAL   x
    LOCAL   aMenu, nMenuLen
    LOCAL   nOption, nOldOption
    LOCAL   nValue
    LOCAL   nCursor
    LOCAL   nKey
    LOCAL   nWidth
    LOCAL   lOverFlow
    LOCAL   cMark
    LOCAL   lDraw
    LOCAL   nOffset
    LOCAL   nPos, nOldPos
    LOCAL   cOldHelp
    LOCAL   lMore := TRUE


    /*
    *   Turn the cursor off.
    */
    nCursor := SetCursor( SC_NONE )


    /*
    *   Menu loop.
    */

    WHILE ( lMore )

        /*
        *   Init.
        */

        aMenu := {}
        nLen := Len( aOptions )
        nWidth := 0

        /*
        *   Check to make sure all options are available.
        */
        FOR x := 1 TO nLen

            IF ( Eval( aOptions[x,2] ) == TRUE )

                Aadd( aMenu, { aOptions[x,1], aOptions[x,3] } )

                /*
                *   Check longest option.
                */

                IF ( nWidth < Len( aOptions[x,1] ) )

                    nWidth := Len( aOptions[x,1] )

                ENDIF

            ENDIF

        NEXT

        /*
        *   Revise the new length.
        */

        nLen := Len( aMenu )

        IF ( nLen > nMaxOptions )

            nMenuLen := nMaxOptions
            lOverFlow := TRUE

        ELSE

            nMenuLen := nLen
            lOverFlow := FALSE

        ENDIF

        /*
        *   Check width against title.
        */

        IF ( nWidth < Len( cTitle ) )

            nWidth := Len( cTitle )

        ENDIF

        /*
        *   Display a Standard Menu Box.
        */

        aScreen := MenuBox( cTitle, nRow, nCol, nMenuLen , nWidth )

        /*
        *   Draw the help here.
        */

        HelpLine( cHelp )

        /*
        *   Initialise all counters.
        */


        nOldOption := 0
        nOldPos := 0
        lDraw := TRUE
        nOption := nSelect

        IF ( nOption > nMenuLen )

            nPos := nMenuLen
            nOffset := nOption - nMenuLen

        ELSE

            nPos := nOption
            nOffset := 0

        ENDIF

        /*
        *   Loop here till we escape.
        */

        DO WHILE ( nMenuLen > 0 )   //  Make sure we have at least one option

            /*
            *   Do we redraw the options?
            */

            IF ( lDraw )

                FOR x := 1 TO nMenuLen

                    SetPos( nRow+2+x, nCol+3 )
                    DispOut( Replicate( " ", nWidth ), Pub( PUB_CLR_BOX_INNER ) )

                    SetPos( nRow+2+x, nCol+3 )
                    DispOut( aMenu[x+nOffset,1], Pub( PUB_CLR_BOX_INNER ) )

                NEXT

                lDraw := FALSE
                nOldOption := 0
                nOldPos := 0

            ENDIF

            /*
            *   Option has changed?
            */

            IF ( nOldOption != nOption )

                IF ( nOldPos > 0  .AND.  nOldOption > 0 )

                    SetPos( nRow+2+nOldPos, nCol+3 )
                    DispOut( aMenu[nOldOption,1], Pub( PUB_CLR_BOX_INNER ) )

                ENDIF

                SetPos( nRow+2+nPos, nCol+3 )
                DispOut( aMenu[nOption,1], Pub( PUB_CLR_BOX_MENUBAR ) )

                nOldOption := nOption
                nOldPos := nPos

                /*
                *   Check overflow markers?
                */

                IF ( lOverFlow )

                    cMark = IIF( nOffset > 0, "", " " )
                    SetPos( nRow+2+1, nCol+1 )
                    DispOut( cMark, Pub( PUB_CLR_BOX_INNER ) )

                    cMark = IIF( nLen - nOffset > nMenuLen, "", " " )
                    SetPos( nRow+2+nMenuLen, nCol+1 )
                    DispOut( cMark, Pub( PUB_CLR_BOX_INNER ) )

                ENDIF

            ENDIF

            /*
            *   Wait till we get a keypress.
            */

            nKey := Inkey(0)

            /*
            *   Process the keypress.
            */

            IF ( nKey == K_ESC )

                lMore := FALSE
                EXIT

            ELSEIF ( nKey == K_ENTER )

                IF ( ValType( aMenu[ nOption,2 ] ) == "B" )

                    /*
                    *   Lolite the current menu.
                    */

                    MenuHiLo( cTitle, nRow, nCol, nMenuLen , nWidth, FALSE )

                    /*
                    *   Pass the option selected as the only parameter.
                    */

                    Eval( aMenu[ nOption,2 ], nOption )

                    /*
                    *   Hilite the current menu.
                    */

                    MenuHiLo( cTitle, nRow, nCol, nMenuLen , nWidth, TRUE )

                    /*
                    *   Need to redisplay the menu.
                    */

                    EXIT

                ENDIF

            ELSEIF ( nKey == K_DOWN )

                IF ( nPos == nMenuLen )

                    IF ( lOverFlow )

                        IF ( nOption < nLen )

                            nOffset++
                            nOption++
                            lDraw := TRUE

                        ENDIF

                    ENDIF

                ELSE

                    nOption++
                    nPos++

                ENDIF

            ELSEIF ( nKey == K_UP )

                IF ( nPos == 1 )

                    IF ( lOverFlow )

                        IF ( nOption > 1 )

                            nOffset--
                            nOption--
                            lDraw := TRUE

                        ENDIF

                    ENDIF

                ELSE

                    nOption--
                    nPos--

                ENDIF

            ELSEIF ( nKey == K_HOME )

                nOption := 1
                nOffset := 0
                lDraw := TRUE
                nPos := 1

            ELSEIF ( nKey == K_END )

                nOption := nLen
                nOffset := nLen - nMenuLen
                lDraw := TRUE
                nPos := nMenuLen

            ELSE

                /*
                *   Check for the first letter, starting at the next option.
                */

                nValue := 0

                IF ( nOption > 1  .AND.  nOption < nLen )

                    IF ( ( nValue := Ascan( aMenu, {|a| Left(a[1],1) == Upper(Chr(nKey))}, nOption+1 ) ) > 0 )

                        nOption := nValue

                    ELSEIF ( ( nValue := Ascan( aMenu, {|a| Left(a[1],1) == Upper(Chr(nKey))} ) ) > 0 )

                        nOption := nValue

                    ENDIF

                ELSE

                    IF ( ( nValue := Ascan( aMenu, {|a| Left(a[1],1) == Upper(Chr(nKey))} ) ) > 0 )

                        nOption := nValue

                    ENDIF

                ENDIF

                IF ( nValue > 0 )

                    IF ( nOption > nMenuLen )

                        nPos := nMenuLen
                        nOffset := nOption - nMenuLen

                    ELSE

                        nPos := nOption
                        nOffset := 0

                    ENDIF

                    lDraw := TRUE

                ENDIF

            ENDIF

        ENDDO

        /*
        *   Release the screen.
        */

        RestScreen( aScreen[1], aScreen[2], aScreen[3], aScreen[4], aScreen[5] )

        /*
        *   Restore the old help.
        */

        HelpLine()

    ENDDO

    /*
    *   Restore cursor.
    */

    SetCursor( nCursor )

RETURN

/*******************************************************************/

FUNCTION    MenuBox( cTitle, nRow, nCol, nHeight, nWidth )

    LOCAL   cScreen
    LOCAL   nTop := nRow
    LOCAL   nLeft := nCol
    LOCAL   nBottom := nRow+nHeight+3
    LOCAL   nRight := nCol+nWidth+3


    /*
    *   Save the screen.
    */

    cScreen := SaveScreen( nTop, nLeft, nBottom, nRight )

    /*
    *   Draw the box.
    */

    MenuHiLo( cTitle, nRow, nCol, nHeight, nWidth, TRUE )

RETURN { nTop, nLeft, nBottom, nRight, cScreen }

/*******************************************************************/

PROCEDURE   MenuHiLo( cTitle, nRow, nCol, nHeight, nWidth, lHi )

    LOCAL   cScreen
    LOCAL   nTop := nRow
    LOCAL   nLeft := nCol
    LOCAL   nBottom := nRow+nHeight+3
    LOCAL   nRight := nCol+nWidth+3
    LOCAL   x
    LOCAL   cColor := IIF( lHi,Pub(PUB_CLR_BOX_FRAME),Pub(PUB_CLR_BOX_INNER) )


    /*
    *   Draw the box.
    */

    DispBox( nTop, nLeft, nBottom, nRight,  ;
                Pub( PUB_FRAME_DOUBLE ), cColor )

    SetPos( nTop+2, nLeft )
    DispOut( Replicate( "", nRight-nLeft ), cColor )
    SetPos( nTop+2, nLeft )
    DispOut( "", cColor )
    SetPos( nTop+2, nRight )
    DispOut( "", cColor )

    /*
    *   Draw the Title.
    */

    SetPos( nTop+1, nLeft+1 )
    DispOut( Replicate( " ", nWidth+2 ), cColor )

    SetPos( nTop+1, nLeft + ((nRight-nLeft-Len(cTitle))/2) + 1 )
    DispOut( cTitle, cColor )

    /*
    *   Draw the vertical bar - a la Netware style.
    */

    FOR x := 1 TO nHeight
        SetPos( nTop+2+x, nLeft+1 )
        DispOut( " ", cColor )
    NEXT

RETURN

/*******************************************************************/

PROCEDURE   M_Escape()

    /*
    *   Press escape.
    */

    KEYBOARD Chr( K_ESC )

RETURN

/*******************************************************************/

PROCEDURE   M_Quit()

    /*
    *   Outa here.
    */

    CLS
    QUIT

RETURN

/*******************************************************************/

