/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Bfm Index Module.
*
*/
/*******************************************************************/

STATIC  soBrowse
STATIC  slAppend, slKillAppend
STATIC  saIndex
STATIC  snIndexes := 0
STATIC  saIndexes
STATIC  snIdxPtr := 1
STATIC  slMore

/*******************************************************************/

PROCEDURE   M_BfmIndex( nTime )

    LOCAL   nTop := 9
    LOCAL   nLeft := 3
    LOCAL   nBottom := 20
    LOCAL   nRight := 78
    LOCAL   nHeight := nBottom - nTop + 1
    LOCAL   nWidth := nRight - nLeft + 1
    LOCAL   aScreen
    LOCAL   x
    LOCAL   oColumn
    LOCAL   keystroke
    LOCAL   lGotKey



    /*
    *   Check
    */

    IF ( nTime == 0 )

        snIndexes := 0
        saIndexes := {}

    ENDIF

    /*
    *   Initialisation
    */

    slAppend := FALSE
    slKillAppend := FALSE
    keystroke := 0

    /*
    *   Init the array to hold the new record.
    */

    InitBlank()

    /*
    *   Always default to first record.
    */

    snIdxPtr := 1

    /*
    *   Create the Tbrowse Object.
    */

    soBrowse := TBrowseNew( nTop, nLeft, nBottom, nRight )

    /*
    *   Setup browse.
    */

    soBrowse:headSep := ""
    soBrowse:colSep  := "  "
//  soBrowse:footSep := ""

    soBrowse:skipBlock := {|s| Skipped(s)}
    soBrowse:goTopBlock := {|| snIdxPtr := 1}
    soBrowse:goBottomBlock := {|| snIdxPtr := snIndexes}

    soBrowse:colorSpec := Pub( PUB_CLR_BOX_INNER ) + "," +;
                         Pub( PUB_CLR_BOX_MENUBAR )

    /*
    *   Add the columns to the browse.
    */

    oColumn := TBColumnNew( "TYPE",  {|| GetData(1)} )
    oColumn:width := 11
    soBrowse:addColumn( oColumn )

    oColumn := TBColumnNew( "POS",  {|| GetData(2)} )
    oColumn:width := 5
    soBrowse:addColumn( oColumn )

    oColumn := TBColumnNew( "LEN",  {|| GetData(3)} )
    oColumn:width := 5
    soBrowse:addColumn( oColumn )

    oColumn := TBColumnNew( "FLAGS", {|| GetData(4)} )
    oColumn:width := 42
    soBrowse:addColumn( oColumn )


    /*
    *   Draw the border.
    */

    aScreen := MenuBox( "Btrieve File Keys", nTop-3, nLeft-3, nHeight, nWidth )

    /*
    *   Setup Help
    */

    HelpLine( "ENTER To Edit, DEL To Delete, ESC For BFM Menu" )

    /*
    *   The BIG LOOP
    */

    slMore := TRUE
    WHILE (slMore)

        /*
        *   Check for killing the append.
        */

        IF ( slKillAppend )

            /*
            *   Ask user if they want to save the new record.
            */

            Menu( 7, 27, "Save New Field?", "ESC To Cancel", 2, 10, ;
                { { "No       " , {|| TRUE} , {|| M_Save(FALSE) } } , ;
                  { "Yes      " , {|| TRUE} , {|| M_Save(TRUE)  } } , ;
                  { "Cancel   " , {|| TRUE} , {|| M_Escape()    } } } )

            /*
            *   Reset flag.
            */

            slKillAppend := FALSE

            /*
            *   Check
            */

            IF ( ! slAppend )

                /*
                *   Restore the help.
                */

                HelpLine()

                /*
                *   If we have no records, outa here!
                */

                IF ( snIndexes == 0 )

                    EXIT

                ENDIF

            ENDIF

        ENDIF

        /*
        *   Check for an empty file.
        */

        IF ( snIndexes == 0  .AND.  ! slAppend )

            /*
            *   Go into append mode.
            */

            KEYBOARD Chr(K_DOWN)

        ENDIF

        /*
        *   Keystroke will interrupt stabilize
        */

        lGotKey := FALSE

        WHILE ( !soBrowse:stabilize() )

            IF ( (keystroke := Inkey()) != 0 )

                lGotKey := TRUE
                EXIT

            ENDIF

        ENDDO

        /*
        *   Check last keypress.
        */

        IF ( ! slAppend .AND. keystroke == K_DOWN .AND. soBrowse:hitBottom )

            /*
            *   Go into append mode.
            */

            slAppend := TRUE
            InitBlank()

            IF ( snIndexes > 0 )

                soBrowse:down()
                lGotKey := TRUE

            ENDIF

            /*
            *   Setup Help
            */

            HelpLine( "Editing New Record, ESC For Save Options" )

        ENDIF

        /*
        *   Wait for a key if all is well...
        */

        IF ( ! lGotKey )

            keystroke := Inkey(0)

        ENDIF

        /*
        *   The monster case statement...
        */

        DO CASE

        CASE keystroke == K_ESC

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                EXIT

            ENDIF

        CASE keystroke == K_ENTER

            EditCell( 0 )

        CASE keystroke == K_DOWN

            IF ( ! slAppend )

                soBrowse:down()

            ENDIF

        CASE keystroke == K_UP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:up()

            ENDIF

        CASE keystroke == K_PGDN

            IF ( ! slAppend )

                soBrowse:pageDown()

            ENDIF

        CASE keystroke == K_PGUP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:pageUp()

            ENDIF

        CASE keystroke == K_CTRL_PGUP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:goTop()

            ENDIF

        CASE keystroke == K_CTRL_PGDN

            IF ( ! slAppend )

                soBrowse:goBottom()

            ENDIF

        CASE keystroke == K_RIGHT

            soBrowse:right()

        CASE keystroke == K_LEFT

            soBrowse:left()

        CASE keystroke == K_HOME

            soBrowse:home()

        CASE keystroke == K_END

            soBrowse:end()

        CASE keystroke == K_CTRL_LEFT

            soBrowse:panLeft()

        CASE keystroke == K_CTRL_RIGHT

            soBrowse:panRight()

        CASE keystroke == K_CTRL_HOME

            soBrowse:panHome()

        CASE keystroke == K_CTRL_END

            soBrowse:panEnd()

        CASE keystroke == K_DEL

            IF ( ! slAppend )

                /*
                *   Ask user if the want to delete the column or record.
                */

                Menu( 7, 27, "Delete", "ESC To Cancel", 1, 10, ;
                    { { "Column Field " , {|| TRUE} , {|| M_Delete(TRUE)  } } , ;
                      { "Entire Record" , {|| TRUE} , {|| M_Delete(FALSE) } } , ;
                      { "Cancel       " , {|| TRUE} , {|| M_Escape()    } } } )

            ENDIF

        OTHERWISE


            /*
            *   Is this a valid edit keystroke?
            */

            IF ( keystroke >= K_SPACE  .AND.  keystroke <= Asc( "z" ) )

                EditCell( keystroke )

            ENDIF

        ENDCASE

    ENDDO

    /*
    *   Restore the screen.
    */

    RestScreen( aScreen[1], aScreen[2], aScreen[3], aScreen[4], aScreen[5] )

    /*
    *   Restore help.
    */

    HelpLine()

RETURN

/*******************************************************************/

STATIC FUNCTION     Skipped( nRequest )

    LOCAL   nCount := 0


    /*
    *   Special case for appending a record.
    */

    IF ( slAppend )

        /*
        *   Ask me no questions, I'll tell you no lies...
        */

        RETURN  nRequest

    ENDIF


    /*
    *   Standard procedure...
    */

    IF ( snIndexes > 0 )

        IF ( nRequest == 0 )

            /*
            *   Do nothing
            */

        ELSEIF ( nRequest > 0 )

            /*
            *   Forward
            */

            WHILE ( nCount < nRequest )

                IF ( snIdxPtr == snIndexes )

                    EXIT

                ELSE

                    snIdxPtr++

                ENDIF

				nCount++

            ENDDO

        ELSEIF ( nRequest < 0 )

            /*
            *   Backward
            */

            WHILE ( nCount > nRequest )

                IF ( snIdxPtr == 1 )

                    EXIT

                ELSE

                    snIdxPtr--

                ENDIF

				nCount--

            ENDDO

        ENDIF

    ENDIF

RETURN  nCount

/*******************************************************************/

STATIC FUNCTION     GetData( nPos )

    /*
    *   Check if we are appending.
    */

    IF ( slAppend  .OR.  snIndexes == 0 )

        RETURN  saIndex[ nPos ]

    ENDIF

RETURN  saIndexes[ snIdxPtr, nPos ]

/*******************************************************************/

STATIC PROCEDURE    InitBlank()

    /*
    *   Initialise array used to hold new record information.
    */

    saIndex := { Space(11), 0, 0, Space(50) }

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_Save( lSave )


    IF ( lSave )

        /*
        *   Store the new record.
        */

        Aadd( saIndexes, saIndex )

        /*
        *   Increment record count.
        */

        snIndexes++

    ENDIF

    /*
    *   Reset the flags.
    */

    slAppend := FALSE

    /*
    *   Close the menu.
    */

    KEYBOARD Chr(K_ESC)

    /*
    *   Refresh the browse.
    */

    soBrowse:refreshAll()

RETURN

/*******************************************************************/

STATIC PROCEDURE    EditCell( nKey )

    LOCAL   nPos := soBrowse:colPos
    LOCAL   oColumn := soBrowse:getColumn(nPos)
    LOCAL   mVar := Eval(oColumn:block)
    LOCAL   nCursor := SetCursor( SC_NORMAL )
    LOCAL   GetList := {}
    LOCAL   nRow := Row(), nCol := Col()
    LOCAL   cPicture := ""


    /*
    *   Setup Help
    */

    HelpLine( "ENTER To Save Edit, ESC To Abort" )

    /*
    *   Can we edit this cell?
    */

    IF ( nPos == 1 )

        /*
        *   With the key type we give 'em a menu.
        */

        Menu( 7, 43, "Key Types", "Select an Option, ESC To Abort", 1, 10, ;
            { { "BKT_STRING ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_INTEGER", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_FLOAT  ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_DATE   ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_TIME   ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_DECIMAL", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_MONEY  ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_LOGICAL", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_NUMERIC", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_BFLOAT ", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_LSTRING", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_ZSTRING", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_UBIBARY", {|| TRUE} , {|n| M_Key(n)} } , ;
              { "BKT_AUTOINC", {|| TRUE} , {|n| M_Key(n)} } } )

    ELSEIF ( nPos == 4 )

        /*
        *   With the key flags we give 'em a menu.
        */

        Menu( 7, 43, "Key Flags", "Select an Option, ESC To Abort", 1, 10, ;
            { { "BKF_DUPLICATES  ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_MODIFIABLE  ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_NULLVALUE   ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_SEGMENTED   ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_ALTERNATE   ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_DESCENDING  ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_SUPPLEMENTAL", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_EXTENDED    ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_MANUAL      ", {|| TRUE} , {|n| M_Flag(n)} } , ;
              { "BKF_BINARYKEY   ", {|| TRUE} , {|n| M_Flag(n)} } } )

    ELSE

        IF ( ValType(mVar) == "N" )

            cPicture := "99999"

        ENDIF

        IF ( nKey > 0 )
            KEYBOARD Chr(nKey)
        ENDIF

        @ nRow, nCol GET mVar COLOR Pub( PUB_CLR_CELL_EDIT ) PICT cPicture
        READ

        /*
        *   Do we save to the array?
        */

        IF ( LastKey() != K_ESC )

            IF ( slAppend )

                /*
                *   This is a new field.
                *   Just update the array for now.
                */

                saIndex[ nPos ] := mVar

            ELSE

                /*
                *   This is a record field.
                *   Update the record element.
                */

                saIndexes[ snIdxPtr, nPos ] := mVar

            ENDIF

        ENDIF

    ENDIF

    /*
    *   Restore cursor.
    */

    SetCursor( nCursor )

    /*
    *   Restore help.
    */

    HelpLine()

    /*
    *   Refresh the browse.
    */

    soBrowse:refreshCurrent()

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_Key( nType )

    /*
    *   Becuse if the weird numbering, if nType > 12, Add 2.
    */

    IF ( nType > 12 )

        nType += 2

    ENDIF

    /*
    *   Store the chosen data type.
    */

    IF ( slAppend )

        saIndex[ 1 ] := BtrvKName( nType-1 )

        /*
        *   Autmatically put in extended key flag?
        */

        IF ( nType > 2  .AND.  Empty( saIndex[ 4 ] ) )

            saIndex[ 4 ] := "BKF_EXTENDED"

        ENDIF

    ELSE

        saIndexes[ snIdxPtr, 1 ] := BtrvKName( nType-1 )

        /*
        *   Autmatically put in extended key flag?
        */

        IF ( nType > 2  .AND.  Empty( saIndexes[ snIdxPtr, 4 ] ) )

            saIndexes[ snIdxPtr, 4 ] := "BKF_EXTENDED"

        ENDIF

    ENDIF

    /*
    *   Break out of menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_Flag( nType )

    /*
    *   Store the chosen data type.
    */

    IF ( slAppend )

        IF ( Empty( saIndex[ 4 ] ) )

            saIndex[ 4 ] := Alltrim(BtrvKFName( nType-1 ))

        ELSE

            saIndex[ 4 ] += "+" + Alltrim(BtrvKFName( nType-1 ))

        ENDIF

    ELSE

        IF ( Empty( saIndexes[ snIdxPtr, 4 ] ) )

            saIndexes[ snIdxPtr, 4 ] := Alltrim(BtrvKFName( nType-1 ))

        ELSE

            saIndexes[ snIdxPtr, 4 ] += "+" + Alltrim(BtrvKFName( nType-1 ))

        ENDIF

    ENDIF

    /*
    *   Break out of menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

FUNCTION    BfmIndexChk()

    IF ( snIndexes == 0 )

        RETURN  FALSE

    ENDIF

RETURN  TRUE

/*******************************************************************/

STATIC PROCEDURE    M_Delete( lField )

    LOCAL   nPos := soBrowse:colpos


    /*
    *   Init the append array and use it to delete a field.
    */
    InitBlank()

    /*
    *   What do we delete?
    */

    IF ( lField )

        /*
        *   Wipe the column only.
        */

        saIndexes[ snIdxPtr, nPos ] := saIndex[ nPos ]

    ELSE

        /*
        *   Remove the array record.
        */

        Adel( saIndexes, snIdxPtr )
        snIndexes--
        Asize( saIndexes, snIndexes )

        /*
        *   Check the pointer.
        */

        IF ( snIdxPtr > snIndexes )

            IF ( snIndexes > 0 )

                snIdxPtr := snIndexes

            ELSE

                snIdxPtr := 1

            ENDIF

        ENDIF

    ENDIF

    /*
    *   Refresh the browse.
    */

    soBrowse:refreshCurrent()

    /*
    *   Close the menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

FUNCTION    BfmIndexExec( h )

    LOCAL   x


    /*
    *   Heading.
    */

    Fwriteline( h, "[Indexes]" )

    /*
    *   Write out the indexes for the BFM.
    */

    FOR x := 1 TO snIndexes

        FwriteStr ( h, "keyType     =   " )
        FwriteLine( h, AllTrim(saIndexes[x,1]) )

        FwriteStr ( h, "keyPos      =   " )
        FwriteLine( h, AllTrim(Str(saIndexes[x,2])) )

        FwriteStr ( h, "keyLen      =   " )
        FwriteLine( h, AllTrim(Str(saIndexes[x,3])) )

        FwriteStr ( h, "keyFlags    =   " )
        FwriteLine( h, AllTrim(saIndexes[x,4]) )

        FwriteLine( h, "" )

    NEXT

    /*
    *   Some space.
    */

    FwriteLine( h, "" )
    FwriteLine( h, "" )

RETURN  TRUE

/*******************************************************************/

PROCEDURE   M_IdxUpdate( aArray )

    /*
    *   Clone the array.
    */

    saIndexes := Aclone( aArray )
    snIndexes := Len( saIndexes )

RETURN

/*******************************************************************/

