/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Browse Module.
*
*/

/*******************************************************************/

STATIC  shBtrv
STATIC  slAppend, slKillAppend
STATIC  saRecord
STATIC  soBrowse
STATIC  snRecords
STATIC  snIndex

/*******************************************************************/

PROCEDURE   M_Browse()

    LOCAL   h
    LOCAL   cBfm := SelectBfm( 8, 21, Pub( PUB_THE_DIR ) )


    /*
    *   Check.
    */

    IF ( cBfm == NIL )

        RETURN

    ENDIF

    /*
    *   Browse the Btrieve File.
    */

    BrowseBFM( cBfm )

RETURN

/*******************************************************************/

STATIC PROCEDURE    BrowseBFM( cFile )

    LOCAL   nTop := 7
    LOCAL   nLeft := 3
    LOCAL   nBottom := Pub( PUB_MAXROW ) - 2
    LOCAL   nRight := Pub( PUB_MAXCOL ) - 1
    LOCAL   nHeight := nBottom - nTop + 1
    LOCAL   nWidth := nRight - nleft + 1
    LOCAL   aScreen
    LOCAL   oColumn
    LOCAL   h
    LOCAL   cFileName, nMaxLen
    LOCAL   aFields := Array(0)
    LOCAL   cFieldname, nFieldType, nFieldLen, nFieldDec, nFieldPos
    LOCAL   x
    LOCAL   cLine
    LOCAL   keyStroke
    LOCAL   lGotKey
    LOCAL   lMore
    LOCAL   lNewRecord


    /*
    *   Open the BFM.
    */

    IF ( ( h := Fopen( cFile, 0 ) ) <= 0 )

        Msg( 15, 22, "Open Error", "Failed to Open File;" + cFile )
        RETURN

    ENDIF

    /*
    *   Extract the BFM information.
    */

    cLine := FileLine( h, "fileName", FALSE )
    cFileName := AllTrim(Token( cLine, EQUAL, 2 ))

    cLine := FileLine( h, "maxRecLen", FALSE )
    nMaxLen := Val(AllTrim(Token( cLine, EQUAL, 2 )))

    /*
    *   Read in the fields.
    */

    WHILE ( TRUE )

        /*
        *   Check if we can find a fieldname token.
        */

        IF ( ( cLine := FileLine( h, "fieldName", FALSE ) ) == NIL )

            EXIT

        ELSE

            cFieldName := AllTrim(Token( cLine, EQUAL, 2 ))

        ENDIF

        /*
        *   Process the rest of the field information.
        */

        cLine := FileLine( h, "fieldType", FALSE )
        nFieldType := BtrvDType( AllTrim(Token( cLine, EQUAL, 2 )) )

        cLine := FileLine( h, "fieldLen", FALSE )
        nFieldLen  := Val( Token( cLine, EQUAL, 2 ) )

        cLine := FileLine( h, "fieldDec", FALSE )
        nFieldDec  := Val( Token( cLine, EQUAL, 2 ) )

        cLine := FileLine( h, "fieldPos", FALSE )
        nFieldPos  := Val( Token( cLine, EQUAL, 2 ) )

        Aadd( aFields, { cFieldname, nFieldType, nFieldLen, nFieldDec, nFieldPos } )

    ENDDO

    /*
    *   Close the BFM file.
    */

    Fclose( h )

    /*
    *   Create the Btrieve Object.
    */

    shBtrv := btrNew( cFileName, nMaxLen, {|b,e| BtrvError(b,e)} )

    /*
    *   Define the Btrieve Fields.
    */

    btrDefine( shBtrv, aFields )

    /*
    *   Open the Btrieve File.
    */

    btrOpen( shBtrv, 0 )

    /*
    *   Get record count.
    */

    snRecords := btrRecordCount( shBtrv )

    /*
    *   Position on the first record.
    */

    IF ( snRecords > 0 )

        btrGetFirst( shBtrv, DEFAULT )

    ELSE

        InitBlank( aFields )

    ENDIF

    /*
    *   Create the Tbrowse Object.
    */

    soBrowse := TBrowseNew( nTop, nLeft, nBottom, nRight )

    /*
    *   Setup browse.
    */

    soBrowse:headSep := ""
    soBrowse:colSep  := "  "
//  soBrowse:footSep := ""

    soBrowse:skipBlock := {|s| Skipped(s)}
    soBrowse:goTopBlock := {|| btrGetFirst( shBtrv, DEFAULT )}
    soBrowse:goBottomBlock := {|| btrGetLast( shBtrv, DEFAULT )}

    soBrowse:colorSpec := Pub( PUB_CLR_BOX_INNER ) + "," +;
                         Pub( PUB_CLR_BOX_MENUBAR )

    /*
    *   Add the columns to the browse.
    */

    FOR x := 1 TO Len( aFields )

        /*
        *   Check for fields greater than screen cols.
        */

        cFieldName := aFields[ x,1 ]
        oColumn := TBColumnNew( cFieldname, NIL )

        IF ( aFields[ x, 3 ] <= Pub( PUB_MAXCOL ) )

            /*
            *   Late evaluation of codeblock essential here.
            */

            oColumn:block := &("{|| GetData('" + Str( x ) + "')}")

            /*
            *   Make sure strings are the right length.
            */

            IF ( aFields[ x,2 ] == BDT_ZSTRING  .OR. ;
                 aFields[ x,2 ] == BDT_STRING )

                oColumn:width := aFields[ x,3 ]

            ENDIF

        ELSE

            oColumn:block := {|| "<MEMO>"}

        ENDIF

        soBrowse:addColumn( oColumn )

    NEXT

    /*
    *   Draw the border.
    */

    aScreen := MenuBox( cFileName, nTop-3, nLeft-3, nHeight, nWidth )

    /*
    *   Setup Help
    */

    HelpLine( "ENTER To Edit, ESC To Quit Browsing, F1 for Options" )

    /*
    *   Initialisation
    */

    snIndex := 1
    slAppend := FALSE
    slKillAppend := FALSE
    keystroke := 0

    lMore := TRUE
    WHILE (lMore)

        /*
        *   Check for killing the append.
        */

        IF ( slKillAppend )

            /*
            *   Restore help.
            */

            HelpLine()

            /*
            *   Ask user if the want to save the new record.
            */

            Menu( 7, 27, "Save New Record?", "ESC To Cancel", 2, 10, ;
                { { "No       " , {|| TRUE} , {|| M_Save(FALSE) } } , ;
                  { "Yes      " , {|| TRUE} , {|| M_Save(TRUE)  } } , ;
                  { "Cancel   " , {|| TRUE} , {|| M_Escape()    } } } )

            /*
            *   Reset flag.
            */

            slKillAppend := FALSE

            /*
            *   Check, if user declined to save the first record.
            */

            IF ( snRecords == 0 )

                EXIT

            ENDIF

        ENDIF

        /*
        *   Keystroke will interrupt stabilize
        */

        lGotKey := FALSE

        WHILE ( !soBrowse:stabilize() )

            IF ( (keystroke := Inkey()) != 0 )

                lGotKey := TRUE
                EXIT

            ENDIF

        ENDDO

        /*
        *   Check for an empty file.
        */

        IF ( snRecords == 0 )

            /*
            *   Go into append mode.
            */

            keystroke := K_DOWN
            soBrowse:hitBottom := TRUE

        ENDIF

        /*
        *   Check last keypress.
        */

        IF ( ! slAppend .AND. keystroke == K_DOWN .AND. soBrowse:hitBottom )

            /*
            *   Go into append mode.
            */

            slAppend := TRUE
            InitBlank( aFields )

            IF ( snRecords > 0 )

                soBrowse:down()
                KEYBOARD Chr(1)

            ENDIF

            /*
            *   Setup Help
            */

            HelpLine( "Editing New Record, ESC For Save Options" )

        ENDIF

        /*
        *   Wait for a key if all is well...
        */

        IF ( ! lGotKey )

            keystroke := Inkey(0)

        ENDIF

        /*
        *   The monster case statement...
        */

        DO CASE

        CASE keystroke == K_F1

            Menu( 7, 27, "HotKey Options", "Select or ESC To Close", 2, 10, ;
                { { "F1     Help Menu   " , {|| TRUE} , {|| TRUE         } } , ;
                  { "F2     Change Index" , {|| TRUE} , {|| M_Index(TRUE)} } } )

        CASE keystroke == K_F2

            M_Index( FALSE )

        CASE keystroke == K_ESC

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                EXIT

            ENDIF

        CASE keystroke == K_ENTER

            EditCell( 0 )

        CASE keystroke == K_DOWN

            IF ( ! slAppend )

                soBrowse:down()

            ENDIF

        CASE keystroke == K_UP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:up()

            ENDIF

        CASE keystroke == K_PGDN

            IF ( ! slAppend )

                soBrowse:pageDown()

            ENDIF

        CASE keystroke == K_PGUP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:pageUp()

            ENDIF

        CASE keystroke == K_CTRL_PGUP

            IF ( slAppend )

                slKillAppend := TRUE

            ELSE

                soBrowse:goTop()

            ENDIF

        CASE keystroke == K_CTRL_PGDN

            IF ( ! slAppend )

                soBrowse:goBottom()

            ENDIF

        CASE keystroke == K_RIGHT

            soBrowse:right()

        CASE keystroke == K_LEFT

            soBrowse:left()

        CASE keystroke == K_HOME

            soBrowse:home()

        CASE keystroke == K_END

            soBrowse:end()

        CASE keystroke == K_CTRL_LEFT

            soBrowse:panLeft()

        CASE keystroke == K_CTRL_RIGHT

            soBrowse:panRight()

        CASE keystroke == K_CTRL_HOME

            soBrowse:panHome()

        CASE keystroke == K_CTRL_END

            soBrowse:panEnd()

        OTHERWISE


            /*
            *   Is this a valid edit keystroke?
            */

            IF ( keystroke >= K_SPACE  .AND.  keystroke <= Asc( "z" ) )

                EditCell( keystroke )

            ENDIF

        ENDCASE

    ENDDO

    /*
    *   Restore the screen.
    */

    RestScreen( aScreen[1], aScreen[2], aScreen[3], aScreen[4], aScreen[5] )

    /*
    *   Restore help.
    */

    HelpLine()

    /*
    *   Close & Kill the Btrieve Object.
    */

    btrClose( shBtrv )
    btrKill( shBtrv )

RETURN

/*******************************************************************/

STATIC FUNCTION Skipped( nRequest )

    LOCAL   nCount := 0


    /*
    *   Special case for appending a record.
    */

    IF ( slAppend )

        /*
        *   Ask me no questions, I'll tell you no lies...
        */

        RETURN  nRequest

    ENDIF


    /*
    *   Standard procedure...
    */

    IF ( snRecords > 0 )

        IF ( nRequest == 0 )

            /*
            *   Do nothing
            */

        ELSEIF ( nRequest > 0 )

            /*
            *   Forward
            */

            WHILE ( nCount < nRequest )

                IF ( btrGetNext( shBtrv ) == BSC_EOF )

                    EXIT

                ENDIF

				nCount++

            ENDDO

        ELSEIF ( nRequest < 0 )

            /*
            *   Backward
            */

            WHILE ( nCount > nRequest )

                IF ( btrGetPrev( shBtrv ) == BSC_EOF )

                    EXIT

                ENDIF

				nCount--

            ENDDO

        ENDIF

    ENDIF

RETURN  nCount

/*******************************************************************/

FUNCTION    GetData( cFieldPos )

    LOCAL   nPos := Val( cFieldPos )


    /*
    *   Check if we are appending.
    */

    IF ( slAppend  .OR.  snRecords == 0 )

        RETURN  saRecord[ nPos ]

    ENDIF

RETURN  btrFieldGet(shBtrv, nPos)

/*******************************************************************/

STATIC PROCEDURE    InitBlank( aFields )

    LOCAL   x, nType, nLen := Len( aFields )


    /*
    *   Initialise array used to hold new record information.
    */

    saRecord := Array(0)

    FOR x := 1 TO nLen

        nType := aFields[ x, 2 ]

        DO CASE

        CASE ( nType == BDT_INTEGER )

            Aadd( saRecord, 0 )

        CASE ( nType == BDT_ZSTRING )

            IF ( aFields[x,3] > Pub(PUB_MAXCOL ) )

                Aadd( saRecord, "<MEMO>" )

            ELSE

                Aadd( saRecord, Replicate( " ", aFields[x,3]-1 ) )

            ENDIF

        CASE ( nType == BDT_STRING )

            IF ( aFields[x,3] > Pub(PUB_MAXCOL ) )

                Aadd( saRecord, "<MEMO>" )

            ELSE

                Aadd( saRecord, Replicate( " ", aFields[x,3] ) )

            ENDIF

        CASE ( nType == BDT_DATE )

            Aadd( saRecord, CToD( "  /  /  " ) )

        CASE ( nType == BDT_LOGICAL )

            Aadd( saRecord, .F. )

        CASE ( nType == BDT_FLOAT )

            Aadd( saRecord, 0.00 )

        CASE ( nType == BDT_LONG )

            Aadd( saRecord, 0 )

        CASE ( nType == BDT_DOUBLE )

            Aadd( saRecord, 0.00 )

        CASE ( nType == BDT_BFLOAT )

            Aadd( saRecord, 0.00 )

        OTHERWISE

            Aadd( saRecord, "N/A" )

        ENDCASE

    NEXT

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_Save( lSave )


    IF ( lsave )

        /*
        *   Store the new record.
        */

        btrSetRecord( shBtrv, saRecord )

        /*
        *   Insert the new record.
        */

        btrInsert( shBtrv, DEFAULT )

        /*
        *   Increment record count.
        */

        snRecords++

    ENDIF

    /*
    *   Reset the flags.
    */

    slAppend := FALSE

    /*
    *   Refresh the browse.
    */

    soBrowse:refreshAll()

    /*
    *   Close the menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

STATIC PROCEDURE    EditCell( nKey )

    LOCAL   nPos := soBrowse:colPos
    LOCAL   oColumn := soBrowse:getColumn(nPos)
    LOCAL   mVar := Eval(oColumn:block)
    LOCAL   nCursor := SetCursor( SC_NORMAL )
    LOCAL   GetList := {}
    LOCAL   nRow := Row(), nCol := Col()


    /*
    *   Setup Help
    */

    HelpLine( "ENTER To Save Edit, ESC To Abort" )

    /*
    *   OK, edit the current cell.
    */

    IF ( ValType(mVar) == "C"  .AND.  mVar == "<MEMO>" )

        KEYBOARD Chr(K_ESC)
        Inkey(0)

    ELSE

        IF ( nKey > 0 )
            KEYBOARD Chr(nKey)
        ENDIF

        @ nRow, nCol GET mVar COLOR Pub( PUB_CLR_CELL_EDIT )
        READ

    ENDIF

    /*
    *   Do we save to the array?
    */

    IF ( LastKey() != K_ESC )

        IF ( slAppend )

            /*
            *   This is a new field.
            *   Just update the array for now.
            */

            saRecord[ nPos ] := mVar

        ELSE

            /*
            *   This is a record field.
            *   Update the record now.
            */

            btrFieldSet( shBtrv, nPos, mVar )
            btrUpdate( shBtrv, DEFAULT )

        ENDIF

    ENDIF

    /*
    *   Restore cursor.
    */

    SetCursor( nCursor )

    /*
    *   Restore help.
    */

    HelpLine()

    /*
    *   Refresh the browse.
    */

    soBrowse:refreshCurrent()

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_Index( lHelp )

    LOCAL   aKeys := btrKeyInfo( shBtrv )
    LOCAL   nIndex := btrGetIndex( shBtrv )
    LOCAL   x
    LOCAL   aMenu := {}
    LOCAL   cType := "1,"


    /*
    *   Extract the index information into a menu.
    */

    FOR x := 1 TO Len( aKeys )

        /*
        *   Build the key prompt
        */

        cType += BtrvKName( aKeys[x,3] )

        /*
        *   Check for segmented index.
        */

        IF ( IsBit( aKeys[x,4], BKF_SEGMENTED ) )

            cType += ";"
            LOOP

        ELSE

            cType += ",P=" + StrZero( aKeys[x,1], 3 )
            cType += ",L=" + StrZero( aKeys[x,2], 3 )

        ENDIF

        /*
        *   Add the index to the menu.
        */

        Aadd( aMenu, { cType, {|| TRUE }, {|n| M_SetIndex( n )} } )

        cType := Ltrim(Str(x+1)) + ","

    NEXT

    /*
    *   Display the menu.
    */

    Menu( 7, 1, "Indexes", "ENTER To Select, ESC To Exit", nIndex, 10, aMenu )

    /*
    *   Close the help menu?
    */

    IF ( lHelp )

        KEYBOARD Chr(K_ESC)

    ENDIF

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_SetIndex( nIndex )

    /*
    *   Set the new index.
    */

    btrSetIndex( shBtrv, nIndex )

    /*
    *   Refresh the browse.
    */

    soBrowse:configure()

    /*
    *   Close the index menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

