/*******************************************************************/
#include "bfm.ch"
/*
*   Btrieve File Manager
*   Copyright (c) Consult Computers P/L
*   All Rights Reserved, 1995.
*
*   Create Module.
*
*/
/*******************************************************************/

PROCEDURE   M_Create()

    /*
    *   Create Menu.
    */

    Menu( 11, 18, "Create", "ESC To Exit Menu", 1, 10 , ;
        { { "BFM Control File", {|| TRUE}, {|| M_NewBfm()} },  ;
          { "Btrieve File    ", {|| TRUE}, {|| M_NewBtr()} } } )

RETURN

/*******************************************************************/

PROCEDURE   M_OldBfm()

    /*
    *   Call the modify procedure.
    */

    M_ModBfm()

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_ModBfm()

    LOCAL   snFile := 1
    LOCAL   snField:= 1
    LOCAL   snIndex:= 1
    LOCAL   lMore := TRUE


    /*
    *   Modify BFM Menu.
    */

    WHILE ( lMore )

        Menu( 12, 33, "Modify BFM File", "ESC To Exit Menu", 1, 10 , ;
            { { "File           ", {|| TRUE}    , {|| M_BfmFile(snFile++)}      },;
              { "Field          ", {|| TRUE}    , {|| M_BfmField(snField++)}    },;
              { "Index          ", {|| TRUE}    , {|| M_BfmIndex(snIndex++)}    },;
              { "Update         ", {|| BfmChk()}, {|| M_BfmCreate(),snFile:=snField:=snIndex:=0} } } )

        /*
        *   Check if user has done anything.
        */

        IF ( snFile + snField + snIndex > 0 )

            Menu( 12, 25, "Abort BFM Modify?", "ESC To Cancel", 2, 10 , ;
                { { "Yes            ", {|| TRUE}, {|| lMore:=FALSE,M_Escape()} }, ;
                  { "No             ", {|| TRUE}, {|| M_Escape()}              } })

        ELSE

            EXIT

        ENDIF

    ENDDO

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_NewBfm()

    LOCAL   snFile := 0
    LOCAL   snField:= 0
    LOCAL   snIndex:= 0
    LOCAL   lMore := TRUE


    /*
    *   Create BFM Menu.
    */

    WHILE ( lMore )

        Menu( 13, 33, "Create BFM File", "ESC To Exit Menu", 1, 10 , ;
            { { "File           ", {|| TRUE}    , {|| M_BfmFile(snFile++)}      },;
              { "Field          ", {|| TRUE}    , {|| M_BfmField(snField++)}    },;
              { "Index          ", {|| TRUE}    , {|| M_BfmIndex(snIndex++)}    },;
              { "Create         ", {|| BfmChk()}, {|| M_BfmCreate(),snFile:=snField:=snIndex:=0} } } )

        /*
        *   Check if user has done anything.
        */

        IF ( snFile + snField + snIndex > 0 )

            Menu( 15, 25, "Abort BFM Create?", "ESC To Cancel", 2, 10 , ;
                { { "Yes            ", {|| TRUE}, {|| lMore:=FALSE,M_Escape()} }, ;
                  { "No             ", {|| TRUE}, {|| M_Escape()}              } })

        ELSE

            EXIT

        ENDIF

    ENDDO

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_BfmCreate()

    LOCAL   cBfmName := BfmFileName()
    LOCAL   h := Fcreate( cBfmName, FC_NORMAL )


    /*
    *   Create the bfm file.
    */

    IF ( h > 0 )

        IF ( ! BfmFileExec(h)   .OR.;
             ! BfmFieldExec(h)  .OR.;
             ! BfmIndexExec(h) )

            Msg( 14, 22, cBfmName, "Failed To Update BFM File." )

        ELSE

            /*
            *   We are done.
            */

            Msg( 14, 22, cBfmName, "File Created Successfully." )

        ENDIF

        /*
        *   Close the file.
        */

        Fclose( h )

    ELSE

        Msg( 14, 22, cBfmName, "Failed To Create File." )

    ENDIF

    /*
    *   Close the menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

STATIC PROCEDURE    M_NewBtr()

    LOCAL   cBfmName := SelectBfm( 10, 36, Pub( PUB_THE_DIR ) )
    LOCAL   h
    LOCAL   cLine
    LOCAL   nPos
    LOCAL   cFileName, nFixedLen, nPageSize, nMaxLen
    LOCAL   cFlags, cFlag, nFlags := 0
    LOCAL   nKeyType, nKeyPos, nKeyLen, nKeyFlags, cKeyFlags
    LOCAL   aKeys := {}
    LOCAL   hBtrv
    LOCAL   lCreate := FALSE
    LOCAL   cOption


    /*
    *   Check.
    */

    IF ( cBfmName == NIL )

        RETURN

    ENDIF

    /*
    *   Read in the Bfm file information to create a btrieve file.
    */

    IF ( ( h := Fopen( cBfmName, FC_NORMAL ) ) > 0 )

        /*
        *   Extract the BFM information.
        */

        cLine := FileLine( h, "fileName", FALSE )
        cFileName := AllTrim(Token( cLine, EQUAL, 2 ))

        cLine := FileLine( h, "fixedLen", FALSE )
        nFixedLen := Val(AllTrim(Token( cLine, EQUAL, 2 )))

        cLine := FileLine( h, "maxRecLen", FALSE )
        nMaxLen := Val(AllTrim(Token( cLine, EQUAL, 2 )))

        cLine := FileLine( h, "pageSize", FALSE )
        nPageSize := Val(AllTrim(Token( cLine, EQUAL, 2 )))

        cLine := FileLine( h, "flags", FALSE )
        cFlags := AllTrim(Token( cLine, EQUAL, 2 ))

        /*
        *   Separate out the flag bits.
        */

        WHILE ( ! Empty( cFlags ) )

            IF ( ( nPos := At( PLUS, cFlags ) ) == 0 )

                nFlags += BtrvFFType( AllTrim( cFlags, PLUS, 1 ) )
                EXIT

            ELSE

                nFlags += BtrvFFType( AllTrim(Token( cFlags, PLUS, 1 )) )
                cFlags := Right( cFlags, Len( cFlags ) - nPos )

            ENDIF

        ENDDO

        /*
        *   Read in the indexes.
        */

        WHILE ( TRUE )

            /*
            *   Check if we can find a fieldname token.
            */

            IF ( ( cLine := FileLine( h, "keyType", FALSE ) ) == NIL )

                EXIT

            ELSE

                nKeyType := BtrvKType( AllTrim(Token( cLine, EQUAL, 2 )) )

            ENDIF

            /*
            *   Process the rest of the index information.
            */

            cLine := FileLine( h, "keyPos", FALSE )
            nKeyPos := Val( Token( cLine, EQUAL, 2 ) )

            cLine := FileLine( h, "keyLen", FALSE )
            nKeyLen  := Val( Token( cLine, EQUAL, 2 ) )

            cLine := FileLine( h, "keyFlags", FALSE )
            cKeyFlags  := Token( cLine, EQUAL, 2 )
            nKeyFlags  := 0

            /*
            *   Process the key flags.
            */

            WHILE ( ! Empty( cKeyFlags ) )

                IF ( ( nPos := At( PLUS, cKeyFlags ) ) == 0 )

                    nKeyFlags += BtrvKFType( AllTrim( cKeyFlags, PLUS, 1 ) )
                    EXIT

                ELSE

                    nKeyFlags += BtrvKFType( AllTrim(Token( cKeyFlags, PLUS, 1 )) )
                    cKeyFlags := Right( cKeyFlags, Len( cKeyFlags ) - nPos )

                ENDIF

            ENDDO

            /*
            *   Add to the key array.
            */

            Aadd( aKeys, { nKeyPos, nKeyLen, nKeyFlags, nKeyType } )

        ENDDO

        /*
        *   Close the BFM file.
        */

        Fclose( h )

        /*
        *   Ask user.
        */

        IF ( File( cFileName ) )

            cOption := "Overwrite   "

        ELSE

            cOption := "Create      "

        ENDIF


        Menu( 15, 25, cFileName, "ESC To Cancel", 1, 10 , ;
            { { cOption       , {|| TRUE}, {||lCreate:=TRUE,M_Escape()} }, ;
              { "Abort       ", {|| TRUE}, {||M_Escape()}             } })

        /*
        *   Create the Btrieve Object?
        */

        IF ( lCreate )

            hBtrv := btrNew( cFileName, nMaxLen, {|b,e| BtrvError(b,e)} )

            /*
            *   Create the Btrieve File.
            */

            btrCreate( hBtrv, nFixedLen, nPageSize, nFlags, aKeys )

            /*
            *   Done!
            */

            Msg( 14, 22, cFileName, "Btrieve File Created Successfully." )

        ENDIF

    ELSE

        Msg( 14, 22, cBfmName, "Failed To Open File." )

    ENDIF

    /*
    *   Close menu.
    */

    KEYBOARD Chr(K_ESC)

RETURN

/*******************************************************************/

STATIC FUNCTION BfmChk()

    /*
    *   Check to make sure all parts of the Bfm are valid.
    */

    IF ( ! BfmFileChk() )

        RETURN  FALSE

    ELSEIF ( ! BfmFieldChk() )

        RETURN  FALSE

    ELSEIF ( ! BfmIndexChk() )

        RETURN  FALSE

    ENDIF

RETURN  TRUE

/*******************************************************************/

